<?php

namespace Drupal\imageapi_optimize_lqip\Plugin\ImageAPIOptimizeProcessor;

use Drupal\imageapi_optimize\ConfigurableImageAPIOptimizeProcessorBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Image\ImageFactory;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * LQIP ImageAPI Optimize Processor.
 *
 * @ImageAPIOptimizeProcessor(
 *   id = "lqip",
 *   label = @Translation("LQIP"),
 *   description = @Translation("Create an LQIP version of the image.")
 * )
 */
final class Lqip extends ConfigurableImageAPIOptimizeProcessorBase {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    LoggerInterface $logger,
    ImageFactory $image_factory,
    RequestStack $request_stack,
    FileSystemInterface $file_system
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger, $image_factory);

    $this->requestStack = $request_stack;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('imageapi_optimize'),
      $container->get('image.factory'),
      $container->get('request_stack'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function applyToImage($image_uri) {
    /** @var \Drupal\Core\Image\ImageInterface $source_image */
    $image = $this->imageFactory->get($image_uri, 'gd');
    if ($image && in_array($image->getMimeType(), ['image/gif', 'image/jpeg', 'image/png'])) {
      $temp_destination_uri = $image_uri . '.lqip';

      /** @var \Drupal\system\Plugin\ImageToolkit\GDToolkit $toolkit */
      $toolkit = $image->getToolkit();
      $scaled = ($image->getWidth() > 10) ? imagescale($toolkit->getResource(), 10) : $toolkit->getResource();

      // Save the GDImage to file since there is no way to get the image
      // contents directly from the GDImage class.
      switch ($image->getMimeType()) {
        case 'image/gif':
          imagegif($scaled, $temp_destination_uri);
          break;

        case 'image/jpeg':
          imagejpeg($scaled, $temp_destination_uri);
          break;

        case 'image/png':
          imagepng($scaled, $temp_destination_uri);
          break;
      }

      // Now the file contents can be base 64 encoded.
      $data = file_get_contents($temp_destination_uri);
      $data = base64_encode($data);
      file_put_contents($temp_destination_uri, $data);

      // Since the imageapi_optimize module only works on temporary files, the
      // LQIP will need to be moved manually to it's final destination.
      $this->fileSystem->move($temp_destination_uri, $this->determineLqipUri(), FileSystemInterface::EXISTS_REPLACE);

      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['message'] = [
      '#type' => 'markup',
      '#markup' => $this->t('There are no configuration options for this processor.'),
    ];
    return $form;
  }

  /**
   * Determine LQIP URI.
   */
  protected function determineLqipUri() {

    /** @var \Symfony\Component\HttpFoundation\Request $request */
    $request = $this->requestStack->getCurrentRequest();
    $image_path = $request->getRequestUri();

    // Remove the query string.
    $image_path = strtok($image_path, '?');

    // Extract details from the path. This will always be some kind of image
    // style URL, so safe assumptions can be made:
    //  ^/(.*+?) = public or private path prefix. This can be discarded.
    //  /styles
    //  /([^/]+) = image style name.
    //  /([^/]+) = file system name.
    //  /(.*)$ = the rest of the file name.
    if (preg_match('#^/(.+?)/styles/([^/]+)/([^/]+)/(.*)$#', $image_path, $matches)) {
      $image_path = $matches[3] . '://styles/' . $matches[2] . '/' . $matches[3] . '/' .$matches[4] . '.lqip';

      return $image_path;
    }

    return FALSE;
  }
}
